from enum import Enum

class Values(Enum):
    DRAFT = "DRAFT"
    RETURN = "RETURN"
    MAKER = "MAKER"
    AWAITING_SIGN_A = "AWAITING_SIGN_A"
    AWAITING_SIGN_B = "AWAITING_SIGN_B"
    AWAITING_SIGN_C = "AWAITING_SIGN_C"
    SIGN_A  = "SIGN_A"
    SIGN_B = "SIGN_B"
    SIGN_C = "SIGN_C"



# IMP_NOTE:  THE DRAFT IS DEFAULT VALUE FOR A TRANSITION STARTED PACK 